/**
 * Instrumentation Vuze Plugin
 * 
 * Copyright 2008, Yale Laboratory of Networked Systems
 */

package net.yale.cs.p4p.p2p.azureus.log.listeners;

import java.util.HashMap;
import java.util.Map;

import net.yale.cs.p4p.p2p.azureus.log.XFerLogPlugin;
import net.yale.cs.p4p.p2p.azureus.log.records.PluginDownloadLogs;
import net.yale.cs.p4p.p2p.logs.DownloadLogs.LogRecord;

import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadListener;

public class XFerDownloadListener implements DownloadListener {

	private XFerLogPlugin plugin_;

	private Map<Download, LogRecord> curRecords_ = new HashMap<Download, LogRecord>();

	public XFerDownloadListener(XFerLogPlugin plugin) {
		plugin_ = plugin;

		/* Begin listening on existing downloads */
		for (Download download : plugin_.getPluginInterface().getDownloadManager().getDownloads()) {
			plugin_.getLogger().log("Creating record for existing download " + download.getName());
			downloadStarted(download);
		}
	}

	/* Create log record when we start a download */
	private void downloadStarted(Download download) {
		try {
			curRecords_.put(download, PluginDownloadLogs.MakeLogRecordDownloadStart(plugin_, download));
		} catch (Exception e) {
			plugin_.getLogger().log("Failed to log transfer ended event", e);
		}
	}

	/* Fill in the rest of the log record when we finish a download. Also,
	 * send the log record to our log processing chain. */
	private void downloadStopped(Download download) {

		LogRecord rec = curRecords_.remove(download);
		if (rec == null)
			return;

		try {
			plugin_.getLogProcessChain().push(PluginDownloadLogs.MakeLogRecordDownloadEnd(rec, download));
		} catch (Exception e) {
			plugin_.getLogger().log("Failed to log transfer ended event", e);
		}
	}

	public void positionChanged(Download download, int oldPosition, int newPosition) {}

	public void stateChanged(Download download, int old_state, int new_state) {
		/* A state change tells us when we've started downloading or stopped downloading */
		if (old_state != Download.ST_DOWNLOADING && new_state == Download.ST_DOWNLOADING)
			downloadStarted(download);
		else if (old_state == Download.ST_DOWNLOADING && new_state != Download.ST_DOWNLOADING)
			downloadStopped(download);
	}

	public void cleanup() {
		/* Send pending log records */
		while (!curRecords_.isEmpty())
			downloadStopped(curRecords_.keySet().iterator().next());
	}

}
